"use strict";

var _http = _interopRequireDefault(require("http"));

var _getPort = _interopRequireDefault(require("get-port"));

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

var _nodeHtmlParser = require("node-html-parser");

var _index = _interopRequireDefault(require("./index"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getDocument = async ({
  head = '',
  body = '',
  responseMethod = 'end',
  options
} = {}) => {
  const port = await (0, _getPort.default)();
  const doc = `
    <!doctype html>
    <head>
      ${head}
    </head>
    <html>
      <body>
        ${body}
      </body>
    </html>
  `;

  const server = _http.default.createServer((req, res) => {
    const purify = (0, _index.default)(options);
    purify(req, res);

    if (responseMethod === 'end') {
      res.end(doc);
    }

    if (responseMethod === 'write') {
      res.write(doc);
      res.end();
    }
  });

  server.listen(port);
  const res = await (0, _nodeFetch.default)(`http://127.0.0.1:${port}`);
  const nodeHttpParserOpts = {
    style: true,
    script: true,
    comments: true,
    pre: true
  };
  server.close();
  return {
    original: (0, _nodeHtmlParser.parse)(doc, nodeHttpParserOpts),
    purified: (0, _nodeHtmlParser.parse)(await res.text(), nodeHttpParserOpts)
  };
};

describe('Purify AMP CSS', () => {
  describe.each(['write', 'end'])('response.%s', responseMethod => {
    it('strips unused css from the response', async () => {
      const {
        purified
      } = await getDocument({
        responseMethod,
        head: '<style amp-custom>.yes { background: green; } .no { background: red; }</style>',
        body: '<div class="yes" />'
      });
      const ampCss = purified.querySelector('head').childNodes.find(node => node.tagName === 'style').text;
      expect(ampCss).toEqual('.yes{background:green}');
    });
    it('does not strip anything extra from the document head', async () => {
      const {
        original,
        purified
      } = await getDocument({
        head: `
          <style />
          <meta />
          <script />
          <noscript />
          <base />
        `
      });
      expect(original.toString()).toEqual(purified.toString());
    });
    it('returns the document unchanged if no AMP CSS', async () => {
      const {
        original,
        purified
      } = await getDocument({
        body: '<div class="yes" />'
      });
      expect(original.toString()).toEqual(purified.toString());
    });
    it('does not minify the AMP CSS if minification disabled', async () => {
      const {
        purified
      } = await getDocument({
        responseMethod,
        head: '<style amp-custom>.yes { background: green; }</style>',
        body: '<div class="yes" />',
        options: {
          minify: false
        }
      });
      const ampCss = purified.querySelector('head').childNodes.find(node => node.tagName === 'style').text;
      expect(ampCss).toEqual('.yes {\n  background: green;\n}');
    });
    it('does not remove whitelisted selectors', async () => {
      const {
        purified
      } = await getDocument({
        responseMethod,
        head: '<style amp-custom>.no { background: red; }</style>',
        options: {
          whitelist: ['.no']
        }
      });
      const ampCss = purified.querySelector('head').childNodes.find(node => node.tagName === 'style').text;
      expect(ampCss).toEqual('.no{background:red}');
    });
    describe('debug mode', () => {
      const originalConsoleLog = console.log;
      beforeAll(() => {
        console.log = jest.fn();
      });
      afterAll(() => {
        console.log = originalConsoleLog;
      });
      it('reports how many bytes were removed', async () => {
        await getDocument({
          responseMethod,
          head: '<style amp-custom>.a {}</style>',
          options: {
            debug: true
          }
        });
        expect(console.log).toHaveBeenCalledWith('Purge AMP CSS removed 5 bytes of unused CSS (100.00%)');
      });
      it('report if no AMP CSS found', async () => {
        await getDocument({
          responseMethod,
          options: {
            debug: true
          }
        });
        expect(console.log).toHaveBeenCalledWith('Purge AMP CSS found no <style amp-custom> element');
      });
    });
  });
});